package com.sleepycat.je.evictor;

import com.sleepycat.je.DatabaseException;
import com.sleepycat.je.cleaner.Cleaner;
import com.sleepycat.je.cleaner.FileProcessor;
import com.sleepycat.je.dbi.DatabaseImpl;
import com.sleepycat.je.dbi.MemoryBudget;
import com.sleepycat.je.tree.LN;
import com.sleepycat.je.tree.TreeLocation;

public privileged aspect CleanerEviction {
	/**
	 * Whether the cleaner should participate in critical eviction. Ideally the
	 * cleaner would not participate in eviction, since that would reduce the
	 * cost of cleaning. However, the cleaner can add large numbers of nodes to
	 * the cache. By not participating in eviction, other threads could be kept
	 * in a constant state of eviction and would effectively starve. Therefore,
	 * this setting is currently enabled.
	 */
	static final boolean DO_CRITICAL_EVICTION = true;

	before(Cleaner cleaner) throws DatabaseException :
		execution(void Cleaner.processPendingLN(LN, DatabaseImpl, byte[], byte[], TreeLocation)) && 
		this(cleaner)
	{
		/* Evict before processing each entry. */
		if (DO_CRITICAL_EVICTION) {
			cleaner.env.getEvictor().doCriticalEviction();
		}
	}

	after(FileProcessor fp) throws DatabaseException :
		call(void FileProcessor.hook_beforeProcess())
		&& withincode(boolean FileProcessor.hookr_processFileInternal(..)) && this(fp){
		/* Evict after updating the budget. */
		if (DO_CRITICAL_EVICTION) {
			fp.env.getEvictor().doCriticalEviction();
		}

	}

}
