package com.sleepycat.je.evictor;

import java.util.SortedMap;

import com.sleepycat.je.DatabaseException;
import com.sleepycat.je.cleaner.UtilizationProfile;
import com.sleepycat.je.dbi.CursorImpl;
import com.sleepycat.je.dbi.DatabaseImpl;
import com.sleepycat.je.dbi.MemoryBudget;
import com.sleepycat.je.incomp.INCompressor;
import com.sleepycat.je.recovery.Checkpointer;
import com.sleepycat.je.recovery.Checkpointer.CheckpointReference;
import com.sleepycat.je.tree.BIN;
import com.sleepycat.je.tree.LN;
import com.sleepycat.je.txn.LockType;

//requires evictor and memorybudget!
public privileged aspect CriticalEviction {
	
	
	
	/**
	 * Do a check on whether synchronous eviction is needed.
	 */
	public void Evictor.doCriticalEviction() throws DatabaseException {
		MemoryBudget mb = envImpl.getMemoryBudget();// MB
		long currentUsage = mb.getCacheMemoryUsage();
		long maxMem = mb.getCacheBudget();
		long over = currentUsage - maxMem;

		if (over > mb.getCriticalThreshold()) {
			if (Evictor.DEBUG) {
				System.out.println("***critical detected:" + over);
			}
			doEvict(Evictor.SOURCE_CRITICAL, true /* evictDuringShutdown */);
		}
	}
	
	
	before(Checkpointer cp) throws DatabaseException :
		call(void Checkpointer.hook_checkDeleted(CheckpointReference, SortedMap, boolean, long)) && this(cp){
		
		/* Evict before each operation. */
		cp.envImpl.getEvictor().doCriticalEviction();
	}
	before(UtilizationProfile c) throws DatabaseException :
		call(LN CursorImpl.getCurrentLN(LockType)) && within(UtilizationProfile) && this(c){
		
		/* Evict before each operation. */
		c.env.getEvictor().doCriticalEviction();
	}
	before(INCompressor c) throws DatabaseException :
		execution(BIN INCompressor.searchForBIN(DatabaseImpl, byte[], byte[])) && this(c)
	{
		
		/* Perform eviction before each operation. */
		c.env.getEvictor().doCriticalEviction();
	}
}
