package com.sleepycat.je.evictor;

import com.sleepycat.je.Cursor;
import com.sleepycat.je.DatabaseException;
import com.sleepycat.je.cleaner.UtilizationProfile;
import com.sleepycat.je.dbi.CursorImpl;
import com.sleepycat.je.dbi.DatabaseImpl;
import com.sleepycat.je.tree.FileSummaryLN;
import com.sleepycat.je.txn.Locker;

public privileged aspect CursorEviction {
	
	private boolean CursorImpl.allowEviction = true;
	/**
	 * Disables or enables eviction during cursor operations for an internal
	 * cursor. For example, a cursor used to implement eviction should not
	 * itself perform eviction. Eviction is enabled by default.
	 */
	public void CursorImpl.setAllowEviction(boolean allowed) {
		allowEviction = allowed;
	}
	
	after(CursorImpl cursor) throws DatabaseException:
		(
				execution(void CursorImpl.close()) || 
				execution(void CursorImpl.reset()) || 
				execution(CursorImpl CursorImpl.cloneCursor(boolean, CursorImpl))
		) && this(cursor) {
		
		/* Perform eviction before and after each cursor operation. */
		if (cursor.allowEviction) {
			cursor.database.getDbEnvironment().getEvictor().doCriticalEviction();
		}
	}
	
	
	after() returning (CursorImpl cursor):
		call(CursorImpl.new(DatabaseImpl, Locker)) && 
		withincode(void UtilizationProfile.insertFileSummary(FileSummaryLN, long, int)){
		
		/* Do not allow eviction in a synchronized method. */
		//TODO this might be a candidate for a pattern. but so far only applied in one place.
		cursor.setAllowEviction(false);
	}

}
